#!/usr/bin/perl -w
# By Ryan Castellucci
# http://ryanc.org/
# Anyone is free to do anything with this code, without restriction.
use strict;
use Time::HiRes qw(sleep time);
use Net::Packet;
use Net::Packet::Frame;
use Net::Packet::IPv4;
use Net::Packet::UDP;
use Net::Packet::Layer7;
use Getopt::Long;

my ($rate,$size,$delay,$sleeplen,$host);
my ($l7,$frame);
my $dport = 9;
my $increment = 8;
GetOptions ("rate=s"      => \$rate,
            "size=s"      => \$size,
            "host=s"      => \$host,
            "dport=s"     => \$dport,
            "increment=s" => \$increment);
die "Host not specified!" unless ($host);
# Protocol 17 = UDP;
my $ip_h = Net::Packet::IPv4->new(dst => $host, protocol => 17);
my $udp_h = Net::Packet::UDP->new(dst => $dport);
my $skipped_sleep;
calcdelay();
my $nexttime = time;
# Handle signals
$SIG{'USR1'} = 'inc_rate';
$SIG{'USR2'} = 'dec_rate';

while (1)
{
  #print $data;
  $frame->send;
  $nexttime += $delay;
  $sleeplen = $nexttime - time;
        if ($sleeplen > 0)
        {
    sleep($sleeplen);
                $skipped_sleep = 0;
  } elsif ($skipped_sleep < 1000) {
                $skipped_sleep++;
          print "Warning: Skipped $skipped_sleep sleep cycles, rate $rate may be too fast.\n" if ($skipped_sleep % 100 == 0);
        } elsif ($skipped_sleep == 1000) {
                $skipped_sleep++;
                die "Unable to meet rate $rate!\n";
  }
}
sub calcdelay
{
  die "Packet size must be at least 29 and no more then 1500.\n" if ($size < 29 || $size > 1500);
  die "Rate must be greater then 0.\n" if ($rate <= 0);
  $delay = 1 / (($rate * 1000) / ($size * 8));
  $l7 = Net::Packet::Layer7->new(data => "A" x ($size - 28));
  $frame = Net::Packet::Frame->new(l3 => $ip_h, l4 => $udp_h, l7 => $l7);
  print "Sending at " . $rate . sprintf("Kbps: a %d byte packet every %.6f seconds...\n",$size,$delay);
        $skipped_sleep = 0;
}
sub inc_rate
{
  $rate += $increment;
  calcdelay();
}
sub dec_rate
{
  $rate -= $increment;
  calcdelay();
}
